#include <M5Stack.h>
#include <rom/tjpgd.h> //TJpgDec (C)ChaN, 2012

// ▲▼ SD CARD ▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼
#define SD_CS 4
int fileNameListNum = 0 ;
int maxFileSize = 0 ;

struct {
  char fileName[16] ;
  int fileSize ;
  int w ;
  int h ;
} iconData[60];

// △▽ File List △▽△▽△▽△▽
void fileList( ) {
  File root = SD.open("/ICON");
  if (!root) {
    M5.Lcd.setTextColor(RED);
    M5.Lcd.printf("/ICON Open FAIL") ;
    M5.Lcd.setTextColor(BLACK);
    while (1) ;
  }
  printDirectory(root, 0);
  root.close();
}

void printDirectory(File dir, int numTabs) {
  while (true) {
    File entry =  dir.openNextFile();
    if (!entry) {
      // no more files
      break;
    }
    if (entry.isDirectory()) {
      printDirectory(entry, numTabs + 1);
    } else {
      // files have sizes, directories do not
      strcpy(iconData[fileNameListNum].fileName,entry.name()) ;
      iconData[fileNameListNum].fileSize = entry.size() ;
      fileNameListNum ++ ;
      if (maxFileSize < entry.size()) {
        maxFileSize = entry.size() ;
      }
    }
    entry.close();
  }
}

// ▲▼ JPEG ▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼
typedef struct {
  const void *fp; /* File pointer for input function */
  BYTE *fbuf;     /* Pointer to the frame buffer for output function */
  UINT wfbuf;     /* Width of the frame buffer [pix] */
} IODEV;

UINT in_func(JDEC* jd, BYTE* buff, UINT nbyte)
{
  UINT ret = 0;
  IODEV *dev = (IODEV*)jd->device;
  File *f2 = (File *)dev->fp;
  if (buff) {
    return (UINT)f2->read(buff, nbyte);
  } else {
    // Remove bytes from input stream 
    if(f2->seek(nbyte, SeekCur)){ //seek mode:SeekCur=1(FS.h)
      ret = nbyte;
    }else{
      ret = 0;
    }
  }
  return ret;
}

bool LoadImage(int IconNo) {
  void *work;       /* Pointer to the decompressor work area */
  JDEC jdec;        /* Decompression object */
  JRESULT res;      /* Result code of TJpgDec API */
  IODEV devid;      /* User defined device identifier */

  char fileName[48];
  sprintf(fileName,"/ICON/%s",iconData[IconNo].fileName) ;
  
  bool result = true ;
  File file = SD.open(fileName);
  if(!file){
    M5.Lcd.setTextColor(RED);
    M5.Lcd.println("Failed to open file for reading");
    M5.Lcd.setTextColor(BLACK);
    return false;
  }
  devid.fp = &file;
 
  // Allocate a work area for TJpgDec 
  const int WorkBuffSize = 1024 * 4 ;
  work = malloc(WorkBuffSize);
  // Prepare to decompress 
  res = jd_prepare(&jdec, in_func, work, WorkBuffSize, &devid);
  if (res == JDR_OK) {
    iconData[IconNo].w = jdec.width ;
    iconData[IconNo].h = jdec.height ;
  } else {
    M5.Lcd.setTextColor(RED);
    M5.Lcd.printf("Failed to prepare: rc=%d\n", res);
    M5.Lcd.setTextColor(BLACK);
    result = false ;
  }
  file.close( );
  free(work);             // Discard work area 
  work = NULL ;
  return result ;
}

// 表示スクロール
void DispScrool( ) {
  int y = M5.Lcd.getCursorY();
  if (y >= 232) {
    M5.Lcd.fillScreen(WHITE);
    M5.Lcd.setCursor(0, 0);
  }
}

// ▲▼ SETUP ▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼
void setup() {
    M5.begin();
    M5.Power.begin();

    M5.Lcd.fillScreen(WHITE);
    M5.Lcd.setCursor(0, 0);
    M5.Lcd.setTextColor(BLACK);
    M5.Lcd.setTextSize(1);

    if (!SD.begin(SD_CS)) {
      M5.Lcd.setTextColor(RED);
      M5.Lcd.println("Card failed");
      while (1) ;
    }
    if (SD.exists("/Image.h")) {
      M5.Lcd.setTextColor(RED);
      M5.Lcd.println("/Image.h exist. Remove File");
      M5.Lcd.setTextColor(BLACK);
      SD.remove("/Image.h");
    }

    M5.Lcd.setTextColor(BLUE);
    M5.Lcd.println("Seek Icon Folder") ;
    M5.Lcd.setTextColor(BLACK);
    fileList( ) ;
    M5.Lcd.setTextColor(BLUE);
    M5.Lcd.printf("maxFileSize:%d\r\n",maxFileSize) ;
    M5.Lcd.setTextColor(BLACK);

    char fileName[48];
    uint8_t *fileBuff = (uint8_t *)malloc(maxFileSize+1024) ;
    if (fileBuff == NULL) {
      M5.Lcd.setTextColor(RED);
      M5.Lcd.println("Out of Memory!") ;
      while (1) ;
    }

    for (int i=0;i<fileNameListNum;i++) {
      LoadImage(i) ;
      DispScrool( ) ; 
      M5.Lcd.printf("%s - %dByte %d x %d\r\n", iconData[i].fileName, iconData[i].fileSize,
                                            iconData[i].w , iconData[i].h) ;

      sprintf(fileName,"/ICON/%s",iconData[i].fileName) ;
      File imageFile = SD.open(fileName, FILE_READ) ;
      if (imageFile) {
        long dataPos = 0 ;
        while (imageFile.available()) {
          fileBuff[dataPos] = imageFile.read();
          dataPos ++ ;
        }
        imageFile.close( );

        strcpy(fileName,iconData[i].fileName) ;
        for (int j=0;fileName[j]!='\0';j++) {
          if ( fileName[j] == '.') {
            fileName[j] = '\0' ;
            break ;
          }
        }

        File textFile = SD.open("/Image.h", FILE_APPEND) ;
        if (textFile) {
          textFile.printf("static uint8_t %s[] = { // %s\r\n" , fileName , iconData[i].fileName) ;
          textFile.printf("  0x%02X,0x%02X , // DATA SIZE - %d\r\n" , 
                          iconData[i].fileSize / 256 , iconData[i].fileSize % 256 , iconData[i].fileSize) ;
          textFile.printf("  0x%02X,0x%02X , 0x%02X,0x%02X , // Width:%d x Hight:%d\r\n  " , 
                          iconData[i].w / 256 , iconData[i].w % 256 , iconData[i].h / 256 , iconData[i].h % 256 ,
                          iconData[i].w , iconData[i].h) ;
          textFile.flush( ) ;
          for (dataPos=0;dataPos < iconData[i].fileSize;dataPos++) {
            textFile.printf("0x%02X,",fileBuff[dataPos]) ;
            if ((dataPos+1) % 16 == 0) {
              textFile.printf("\r\n  ") ;
            }
            textFile.flush( ) ;
          }
          
          textFile.println(" } ;") ;
          textFile.println( ) ;
          textFile.flush( ) ;
          delay(500) ;
          textFile.close() ;
          delay(500) ;
        }
      } else {
        M5.Lcd.setTextColor(RED);
        M5.Lcd.printf("Open FAIL:%s\r\n",fileName) ;
        M5.Lcd.setTextColor(BLACK);
      }
    }
    free(fileBuff) ;
}

void loop() {
}
